/************************************************
Modified and Enhanced by Jeremy Paskali
and Ian McCloy!
Final Project
CS 427
Daniel Webster College
************************************************/

#include <windows.h>
#include <time.h>
#include <stdio.h>
#include <gl\Gl.h>
#include <gl\Glu.h>
#include <gl\glut.h>
#include <gl\glaux.h>
#include <string>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include "midi.h"
#include "camera.h"

#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")

using namespace std;


/********************/
/* GLOBAL VARIABLES */
/********************/

// camera and mouse
CCamera gCamera;
Point3 camera, look;
Vector3 up;

// animation
float T = 0.0;
float StartT = 0.0;
float gPaddleX = 0;
float gPaddleY = 0;

float rotateX = 0;
float rotateY = 0;
float rotRateX = 0;
float rotRateY = 0;

float DeltaMouseX = 0.0;
float DeltaMouseY = 0.0;
int NumberOfLives = 4;
float PI = 3.14159;
float theta = 0;



// light variables
GLfloat light_position[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat global_ambient[] = {0.4, 0.4, 0.4};
GLfloat global_darkness[] = {0.0, 0.0, 0.0};


// colors
GLfloat lgray[] = { 0.8, 0.8, 0.8, 1.0 };
GLfloat gray[]	= { 0.5, 0.5, 0.5, 1.0 };
GLfloat black[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat red[]	= { 1.0, 0.1, 0.1, 1.0 };
GLfloat white[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat blue[]	= { 0.1, 0.1, 1.0, 1.0 };
GLfloat green[]	= { 0.1, 1.0, 0.1, 1.0 };
GLfloat pink[]	= { 1.0, 0.4, 1.0, 1.0 };
GLfloat yellow[]= { 1.0, 1.0, 0.1, 1.0 };
GLfloat orange[]= { 1.0, 0.7, 0.1, 1.0 };
GLfloat brown[] = { 0.5, 0.2, 0.1, 1.0 };

// This will reference to our texture data stored with OpenGL
#define MAX_TEXTURES 1
UINT g_Texture[MAX_TEXTURES];

// display list references
GLuint cubeDL;
GLuint boundsDL;
GLuint paddleDL;
GLuint padShadowDL;
GLuint DaBall;
GLuint BricksDL;

// Ball vector info 
double initialx = .0000;					//the inital values for the ball
double initialy = .0001;
double initialz = -.06;
double Menux = .0005;						//the values used for the ball in the opening screen
double Menuy = .0009;
Point3 Ball(0.0, 0.0, 9.5);					//the ball location and vector which we add on
Point3 Ballv(Menux, Menuy, initialz);

// Paddle config
float gPaddleSize = 0.2;
float gPadNear = 9.9;
float gPadFar = 9.85;

// Window Parameters
float gWidth;
float gHeight;

// Linked List of bricks
struct brick;
typedef brick *brickptr;
struct brick
{
	float color[4];
	float locX;
	float locY;
	float locZ;
	bool hit;
	brickptr next;
};

// Brick head of linked list				//We store our bricks in a linked list
brickptr BrickLevelOne = NULL;

// Brick settings
float gBrickWidth = 0.5;
float gBrickHeight = 0.5;
float gBrickDepth = 0.5;

// Game controls
bool gPause = false;
bool gGuide = false;
bool gHold = false;
bool gStop = true;
bool gMenu = true;
bool hit = false;							//have we just hit a ball?
bool loading = false;						//are we loading?
bool inital = true;							//did we just start?

unsigned long int gPlayerScore = 0;
unsigned long int gPlayerCombo = 0;

int CurrentLevel = 0; 

/***********************/
/* FUNCTION PROTOTYPES */
/***********************/
void Keyboard(unsigned char theKey, int mouseX, int mouseY);
void userKey(unsigned char theKey);
void userHelp();
void Idle();
void mySpecial(int theKey, int x, int y);
void myReshape(int width, int height);
void userInit();		
void PassiveMotion(int x, int y);											
void Motion(int x, int y);
void Mouse(int button, int state, int x, int y);
GLuint CreateLevelBounds();
GLuint CreatePaddle(float size, float pnear, float pfar);
GLuint CreatePaddleShadow(float size, float pnear, float pfar);
GLuint CreateBall();
void FrameRate();					//This keeps the game at the same speed on diffrent systems
void BallCollisionDetection();
void InitBrickSetup();
GLuint DrawBricks(brickptr &head, float height, float width, float depth);
void MusicSelector(int selection);	//Begins playing a midifile
void DisplayMenu();					//Draws the opening menu/screen
void Reset();						//Resets the game
void ResetBricks();					//Resets the bricks, called by reset
void DrawGame();					//Draws the game screen
void NextLevel();					//Goes on to the nest level
void LoadScreen();					//Draws the load screen
bool Check4Win();					


//*********************************************************************
// This is the whole display to the back buffer routine
//*********************************************************************
void myDisplay( void )
{
	// Clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Do not show cursor
	ShowCursor(FALSE);

	// Disable alpha blend temporarily
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	
	//Draw Menu or the game
	if(inital)
	{
		T = glutGet(GLUT_ELAPSED_TIME);
		if( T - StartT > 500 )
		{
			inital = false;
			MusicSelector(1+(rand()%19));
		}
	}
	else if(loading)
		LoadScreen();	
	else if(gMenu)
		DisplayMenu();
	else
	{
		DrawGame();

		//******************************
		// Heads Up Display
		//******************************
		glEnable(GL_LIGHT1);

		glColor3fv(red);
		FrameRate();

		// If game paused
		if( gPause )
		{
			char pause[10] = "";
			sprintf(pause,"Paused");
			glRasterPos3f(-0.1, 0, 9.9);
			for(int x = 0; x < 10; x++)
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, pause[x]);
		}

		// Ball behind the paddle
		char s[50] = "";
		if(Ball.z > gPadNear)
		{
			sprintf(s,"Gir - \"Where's My Moose?\"");

			glRasterPos3f(-0.28, 0.3, 9.9);

			for(int x = 0; x < 50; x++)
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);
		}
		
		glPushMatrix();
			// Lives left
			char lives[3] = "";
			sprintf(lives,"*");
			int x = 0;
			for(int i = 0; i < NumberOfLives; i++)
			{
				for(x = 0; x < 3; x++)
				{
					glRasterPos3f(-0.28+(i*.1), 0.94, 9.9);
					glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, lives[x]);
				}
			}
			// Print score
			if( hit )
				glColor3fv(yellow);
			else
				glColor3fv(red);

			char score[50] = "";
			sprintf(score,"Score: %d", gPlayerScore);
			glRasterPos3f(-1.0, 0.9, 9.9);
			for(x = 0; x < 50; x++)
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, score[x]);
			hit = false;

			//Print level
			sprintf(score,"Level: %i                ", CurrentLevel);
			glRasterPos3f(-1.0, 0.97, 9.9);
			for(x = 0; x < 50; x++)
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, score[x]);

			//Print Combo info
			char combo[30] = "";
			if( gPlayerCombo >= 5 && gPlayerCombo < 10 )
			{
				glColor3fv(blue);
				glRasterPos3f(-1.0, 0.8, 9.9);
				sprintf(combo, "Combo: %d", gPlayerCombo);
				for(int x = 0; x < 30; x++)
					glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10, combo[x]);
			}
			else if( gPlayerCombo >= 10 && gPlayerCombo < 15 )
			{
				glColor3fv(green);
				glRasterPos3f(-1.0, 0.8, 9.9);
				sprintf(combo, "Excellent Combo: %d", gPlayerCombo);
				for(int x = 0; x < 30; x++)
					glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, combo[x]);
			}
			else if( gPlayerCombo >= 15 && gPlayerCombo < 20 )
			{
				glColor3fv(orange);
				glRasterPos3f(-1.0, 0.8, 9.9);
				sprintf(combo, "Mega Combo: %d", gPlayerCombo);
				for(int x = 0; x < 30; x++)
					glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, combo[x]);
			}
			else if( gPlayerCombo >= 20 )
			{
				glColor3fv(pink);
				glRasterPos3f(-1.0, 0.8, 9.9);
				sprintf(combo, "ULTRA COMBO: %d", gPlayerCombo);
				for(int x = 0; x < 30; x++)
					glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, combo[x]);
			}
		glPopMatrix();
		glDisable(GL_LIGHT1);
	}//display menu else

	// Present Back buffer to the front
	glFlush();
	glutSwapBuffers();
}

void DrawGame()
{
	glPushMatrix();
		//If player is past level 10 then start rotating the screen to make it 
		//Amazy Crazy hard!
		if(CurrentLevel >= 10)
			glRotatef(theta,0,0,1);
	

		// Draw the level boundaries
		glCallList(boundsDL);
		
		// Draw the bricks
		glCallList(BricksDL);

		// Draw the Game Ball
		glPushMatrix();
		{
			if( gStop )
			{
				Ball.x = gPaddleX;
				Ball.y = -gPaddleY;
				Ball.z = gPadFar - 0.09;
			}

			glTranslated(Ball.x, Ball.y, Ball.z);
			glLightfv(GL_LIGHT0, GL_POSITION, light_position);
			glEnable(GL_LIGHT1);
			glCallList(DaBall);
			glDisable(GL_LIGHT1);
		}
		glPopMatrix();

		// Draw the guide lines (cheat)
		if( gGuide )
		{
			glColor4fv(gray);
			glBegin(GL_LINES);
				glVertex3d(gPaddleX-.2,-gPaddleY-.2, 0);
				glVertex3d(gPaddleX-.2,-gPaddleY-.2,13); // along the z axis
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(gPaddleX-.2,-gPaddleY+.2, 0);
				glVertex3d(gPaddleX-.2,-gPaddleY+.2,13); // along the z axis
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(gPaddleX+.2,-gPaddleY+.2, 0);
				glVertex3d(gPaddleX+.2,-gPaddleY+.2,13); // along the z axis
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(gPaddleX+.2,-gPaddleY-.2, 0);
				glVertex3d(gPaddleX+.2,-gPaddleY-.2,13); // along the z axis
			glEnd();

			glBegin(GL_LINES);
				glVertex3d(-1,Ball.y,Ball.z);
				glVertex3d( 1,Ball.y,Ball.z); // along the z axis
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(Ball.x,-1,Ball.z);
				glVertex3d(Ball.x, 1,Ball.z); // along the z axis
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(Ball.x,Ball.y,0);
				glVertex3d(Ball.x,Ball.y,10); // along the z axis
			glEnd();

			glBegin(GL_LINE_LOOP);
				glVertex3f( gPaddleX - 0.25, -(gPaddleY + 0.25), 9.9);
				glVertex3f( gPaddleX + 0.25, -(gPaddleY + 0.25), 9.9);
				glVertex3f( gPaddleX + 0.25, -(gPaddleY - 0.25), 9.9);
				glVertex3f( gPaddleX - 0.25, -(gPaddleY - 0.25), 9.9);
			glEnd();
		}

		// Begin blending again
		glDisable(GL_DEPTH_TEST);
		glEnable(GL_BLEND);

		// Draw the game paddle
		glPushMatrix();
		{
			glTranslated(gPaddleX, -gPaddleY, 0);
			glCallList(paddleDL);
		}
		glPopMatrix();
		
		// Draw the bottom game paddle shadow
		glPushMatrix();
		{
			glTranslated(gPaddleX, -0.8, 0);
			glCallList(padShadowDL);
		}
		glPopMatrix();
		// Draw the top game paddle shadow
		glPushMatrix();
		{
			glTranslated(gPaddleX, 0.8, 0);
			glRotated(180, 0, 0, 1);
			glCallList(padShadowDL);
		}
		glPopMatrix();
		// Draw the left game paddle shadow
		glPushMatrix();
		{
			glTranslated(-0.8, -gPaddleY, 0);
			glRotated(270, 0, 0, 1);
			glCallList(padShadowDL);
		}
		glPopMatrix();
		// Draw the right game paddle shadow
		glPushMatrix();
		{
			glTranslated(0.8, -gPaddleY, 0);
			glRotated(90, 0, 0, 1);
			glCallList(padShadowDL);
		}
		glPopMatrix();

		BallCollisionDetection();
	glPopMatrix();
}
//*********************************************************************************
//	Handle the keyboard.
//*********************************************************************************

void Keyboard(unsigned char theKey, int mouseX, int mouseY)
{
	userKey(theKey);
}
//*********************************************************************************
//	User keystroke routine: gets control when user presses a key.
//*********************************************************************************

void userKey(unsigned char theKey)
{
	switch (theKey)
	{
		case '`':
			MusicSelector(1);
			gPause = true;
			break;
		case '1':
			MusicSelector(2);
			gPause = true;
			break;
		case '2':
			MusicSelector(3);
			gPause = true;
			break;
		case '3':
			MusicSelector(4);
			gPause = true;
			break;
		case '4':
			MusicSelector(5);
			gPause = true;
			break;
		case '5':
			MusicSelector(6);
			gPause = true;
			break;
		case '6':
			MusicSelector(7);
			gPause = true;
			break;
		case '7':
			MusicSelector(8);
			gPause = true;
			break;
		case '8':
			MusicSelector(9);
			gPause = true;
			break;
		case '9':
			MusicSelector(10);
			gPause = true;
			break;
		case '0':
			MusicSelector(11);
			gPause = true;
			break;
		case '-':
			MusicSelector(12);
			gPause = true;
			break;
		case '=':
			MusicSelector(13);
			gPause = true;
			break;
		case '~':
			MusicSelector(14);
			gPause = true;
			break;
		case '!':
			MusicSelector(15);
			gPause = true;
			break;
		case '@':
			MusicSelector(16);
			gPause = true;
			break;
		case '#':
			MusicSelector(17);
			gPause = true;
			break;
		case '$':
			MusicSelector(18);
			gPause = true;
			break;
		case '%':
			MusicSelector(19);
			gPause = true;
			break;

		case 'p':
		case 'P':
			gPause = !gPause;
			gPause ? PauseMidi() : ResumeMidi();
			break;

		case 'g':
		case 'G':
			gGuide = !gGuide;

		case 'a':
		case 'A':
			break;

		case 'd':
		case 'D':
			break;

		case 'w':
		case 'W':
			break;

		case 's':
		case 'S':
			break;

		case 'r':
		case 'R':
			break;

		case 'f':
		case 'F':
			break;

		case 'h':
		case 'H':
			break;

		// Escape key exit
		case 27:
			if(!loading && !gMenu)
				Reset();
			else
			{
				StopMidi();
				exit(0);
			}
			break;
	
		default:
			break;

	}
}


//*********************************************************************************
//	User help routine: gets control when user presses the help key. 
//*********************************************************************************

void userHelp()
{
    // Brief explanation on how to trigger everything.
	cout<<"Beginning the game...\nShould not see me in release version\n";
}




// <<<<<<<<<<<<<<<<    main   >>>>>>>>>>>>>>>>>>
void main (int argc, char **argv)
{
	srand( unsigned( time(NULL) ) );
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	// Enter full screen with 85 refresh rate if possible
	glutGameModeString("800x600:32@85");
	if( glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
	{
		glutEnterGameMode();
	}
	else
	{
		// Enter full screen with 32 bpp if possible
		glutGameModeString("800x600:32");
		if( glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
		}
		else
		{
			// Enter full screen with 16 bpp if possible
			glutGameModeString("800x600:16");
			if( glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
			{
				glutEnterGameMode();
			}
			else
			{
				// Cannot establish full screen
				cout<<"Your computer does not allow for 800x600 full screen resolution.\n";
				exit(1);
			}
		}
	}

	glutKeyboardFunc(Keyboard);
	glutDisplayFunc(myDisplay);
	glutIdleFunc(Idle);
	glutSpecialFunc(mySpecial);
	glutReshapeFunc(myReshape);
	glutMouseFunc(Mouse);
	glutMotionFunc(Motion);
	glutPassiveMotionFunc(PassiveMotion);

	userInit();
	userHelp();

	glutMainLoop();
}

// Initializations
void userInit()
{
	StartT = glutGet(GLUT_ELAPSED_TIME);
	// background is black
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	// Set up viewport with camera
	glViewport(0, 0, 800, 600);
	gCamera.setShape( 60, 1.0, 0.1, 200 );
	look.x = 0;
	look.y = 0;
	look.z = 0;
	camera.x = 0;
	camera.y = 0;
	camera.z = 11.7;
	up.x = 0;
	up.y = 1;
	up.z = 0;
	gCamera.set( camera, look, up );

	// Enable Lighting, shading, and material color
	glEnable (GL_LIGHTING);
	glEnable (GL_COLOR_MATERIAL);
	glShadeModel(GL_SMOOTH);

	// Light effects for light 0
	glLightfv (GL_LIGHT0, GL_AMBIENT, black);
    glLightfv (GL_LIGHT0, GL_DIFFUSE, white);
    glLightfv (GL_LIGHT0, GL_SPECULAR, white);
	glLightf  (GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.2);

	// Bright ambient Light for ball and text
	glLightfv (GL_LIGHT1, GL_AMBIENT, white);
	
	// Set up ambient light through out world
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);

	// Enable Lights
	glEnable(GL_LIGHT0);

	// Set up alpha blending
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// Prepare the brick layout
	InitBrickSetup();

	// Set up display lists
	boundsDL = CreateLevelBounds();
	paddleDL = CreatePaddle(gPaddleSize, gPadNear, gPadFar);
	padShadowDL = CreatePaddleShadow(gPaddleSize, gPadNear, gPadFar);
	DaBall = CreateBall();
	BricksDL = DrawBricks(BrickLevelOne, gBrickHeight, gBrickWidth, gBrickDepth);

	// Save the window parameters
	gWidth = glutGameModeGet(GLUT_GAME_MODE_WIDTH);
	gHeight = glutGameModeGet(GLUT_GAME_MODE_HEIGHT);
}

// Idle callback function for animation
void Idle()
{
	// Draw again
	glutPostRedisplay();
}

//*********************************************************************************
//	Handle special keyboard input.
//	Such as F1 through F12, insert, page up, and the arrow keys
//*********************************************************************************

void mySpecial(int theKey, int x, int y)
{
	switch(theKey)
	{
		case GLUT_KEY_F1:
			break;

		case GLUT_KEY_F2:
			break;

		case GLUT_KEY_F3:
			break;

		case GLUT_KEY_F4:
			break;

		case GLUT_KEY_F5:
			break;

		case GLUT_KEY_F6:
			break;

		case GLUT_KEY_F7:
			break;

		case GLUT_KEY_F8:
			break;

		case GLUT_KEY_F10:
			break;

		case GLUT_KEY_F11:
			break;

		case GLUT_KEY_PAGE_UP:
			break;

		case GLUT_KEY_PAGE_DOWN:
			break;

		case GLUT_KEY_HOME:
			break;

		case GLUT_KEY_END:
			break;
	};
}

// Called when the screen is reshaped
void myReshape(int width, int height)
{
	glViewport(0, 0, width, height);
	gCamera.modifyShape(60, 1.0, 0.1, 200);
	glutPostRedisplay();
}

// Called when the mouse is moved and not button is pressed
void PassiveMotion(int x, int y)
{
	float tempX = float(x) / (gWidth / 2.0) - 1;
	float tempY = float(y) / (gHeight / 2.0) - 1;

	static float beforeX = gPaddleX;
	static float beforeY = gPaddleY;

	//Limits mouse movment and scales cordinates down to our 1.0 X 1.0 screen
	if( x > 720 )
	{
		x = 720;
		SetCursorPos(x, y);
		gPaddleX = 1 - gPaddleSize;
	}
	else if( x < 80)
	{
		x = 80;
		SetCursorPos(x, y);
		gPaddleX = gPaddleSize - 1;
	}
	else
		gPaddleX = tempX;

	if( y > 540 )
	{
		y = 540;
		SetCursorPos(x, y);
		gPaddleY = 1 - gPaddleSize;
	}
	else if( y < 60 )
	{
		y = 60;
		SetCursorPos(x, y);
		gPaddleY = gPaddleSize - 1;
	}
	else
		gPaddleY = tempY;

	// Get difference in paddle position from previous movement
	DeltaMouseX = gPaddleX - beforeX;
	DeltaMouseY = gPaddleY - beforeY;

	beforeX = gPaddleX;
	beforeY = gPaddleY;
}

// Called when the mouse is moved and a button is pressed
void Motion(int x, int y)
{
	PassiveMotion(x, y);
}

// Called when the mouse is clicked
void Mouse(int button, int state, int x, int y)
{
	if( button == GLUT_LEFT_BUTTON && state == GLUT_DOWN )
	{
		if(gMenu)
		{
			gMenu = false;
			NextLevel();
		}
		else if( gStop )
		{
			gStop = false;
		}
	}
}

// This function draws the walls of the level
GLuint CreateLevelBounds()
{
	GLuint tempDL = glGenLists(1);

	glNewList(tempDL, GL_COMPILE);
	{
		int y;
		int x;
		int z;

		// Back Wall
		glColor3fv(gray);
		glBegin(GL_QUADS);
		{
			for( y = 10; y > -10; y -= 2)
			{
				for( x = -10; x < 10; x += 2)
				{
					glNormal3f( 0, 0, 1);
					glVertex3f( (float)x / 10.0, (float)y / 10.0, 0);
					glVertex3f( (float)x / 10.0, (float)y / 10.0 - 0.2, 0);
					glVertex3f( (float)x / 10.0 + 0.2, (float)y / 10.0 - 0.2, 0);
					glVertex3f( (float)x / 10.0 + 0.2, (float)y / 10.0, 0);
				}
			}
		}
		glEnd();

		// Top Wall
		glBegin(GL_QUADS);
		{
			for( z = 0; z < 100; z += 2)
			{
				for( x = -10; x < 10; x += 2)
				{
					glNormal3f( 0, -1, 0);
					glVertex3f( (float)x / 10.0, 1, (float)z / 10.0);
					glVertex3f( (float)x / 10.0, 1, (float)z / 10.0 + 0.2);
					glVertex3f( (float)x / 10.0 + 0.2, 1, (float)z / 10.0 + 0.2);
					glVertex3f( (float)x / 10.0 + 0.2, 1, (float)z / 10.0);
				}
			}
		}
		glEnd();

		// Left Wall
		glBegin(GL_QUADS);
		{
			for( z = 0; z < 100; z += 2)
			{
				for( y = 10; y > -10; y -= 2)
				{
					glNormal3f( 1, 0, 0);
					glVertex3f(-1, (float)y / 10.0, (float)z / 10.0);
					glVertex3f(-1, (float)y / 10.0, (float)z / 10.0 + 0.2);
					glVertex3f(-1, (float)y / 10.0 - 0.2, (float)z / 10.0 + 0.2);
					glVertex3f(-1, (float)y / 10.0 - 0.2, (float)z / 10.0);
				}
			}
		}
		glEnd();

		// Bottom Wall
		glBegin(GL_QUADS);
		{
			for( z = 0; z < 100; z += 2)
			{
				for( x = -10; x < 10; x += 2)
				{
					glNormal3f( 0, 1, 0);
					glVertex3f( (float)x / 10.0, -1, (float)z / 10.0);
					glVertex3f( (float)x / 10.0 + 0.2, -1, (float)z / 10.0);
					glVertex3f( (float)x / 10.0 + 0.2, -1, (float)z / 10.0 + 0.2);
					glVertex3f( (float)x / 10.0, -1, (float)z / 10.0 + 0.2);
				}
			}
		}
		glEnd();

		// Right Wall
		glBegin(GL_QUADS);
		{
			for( z = 0; z < 100; z += 2)
			{
				for( y = 10; y > -10; y -= 2)
				{
					glNormal3f(-1, 0, 0);
					glVertex3f( 1, (float)y / 10.0, (float)z / 10.0);
					glVertex3f( 1, (float)y / 10.0 - 0.2, (float)z / 10.0);
					glVertex3f( 1, (float)y / 10.0 - 0.2, (float)z / 10.0 + 0.2);
					glVertex3f( 1, (float)y / 10.0, (float)z / 10.0 + 0.2);
				}
			}
		}
		glEnd();
	}
	glEndList();

	return tempDL;
}

// This funciton draws the game paddle the user controls
GLuint CreatePaddle(float size, float pnear, float pfar)
{
	GLuint tempDL = glGenLists(1);

	glNewList(tempDL, GL_COMPILE);
	{
		GLfloat alpha[] = {0.2, 0.1, 0.8, 0.3};
		glColor4fv(alpha);

		// Side away from user
		glBegin(GL_QUADS);
		{
			glNormal3f( 0.0, 0.0,-1.0);
			glVertex3f(-size, size, pfar);
			glVertex3f(-size,-size, pfar);
			glVertex3f( size,-size, pfar);
			glVertex3f( size, size, pfar);
		}
		glEnd();

		// Left side
		glBegin(GL_QUADS);
		{
			glNormal3f(-1.0, 0.0, 0.0);
			glVertex3f(-size, size, pnear);
			glVertex3f(-size, size, pfar);
			glVertex3f(-size,-size, pfar);
			glVertex3f(-size,-size, pnear);
		}
		glEnd();

		// Right side
		glBegin(GL_QUADS);
		{
			glNormal3f( 1.0, 0.0, 0.0);
			glVertex3f( size, size, pnear);
			glVertex3f( size,-size, pnear);
			glVertex3f( size,-size, pfar);
			glVertex3f( size, size, pfar);
		}
		glEnd();

		// Top side
		glBegin(GL_QUADS);
		{
			glNormal3f( 0.0, 1.0, 0.0);
			glVertex3f(-size, size, pfar);
			glVertex3f(-size, size, pnear);
			glVertex3f( size, size, pnear);
			glVertex3f( size, size, pfar);
		}
		glEnd();

		// Bottom side
		glBegin(GL_QUADS);
		{
			glNormal3f( 0.0,-1.0, 0.0);
			glVertex3f(-size,-size, pnear);
			glVertex3f(-size,-size, pfar);
			glVertex3f( size,-size, pfar);
			glVertex3f( size,-size, pnear);
		}
		glEnd();

		// Side facing user
		glBegin(GL_QUADS);
		{
			glNormal3f( 0.0, 0.0, 1.0);
			glVertex3f(-size, size, pnear);
			glVertex3f(-size,-size, pnear);
			glVertex3f( size,-size, pnear);
			glVertex3f( size, size, pnear);
		}
		glEnd();
	}
	glEndList();

	return tempDL;
}

// This function draws the shadow for the game paddle
// this one is designed so that the normal is facing
// the positive y-axis
GLuint CreatePaddleShadow(float size, float pnear, float pfar)
{
	GLuint tempDL = glGenLists(1);

	glNewList(tempDL, GL_COMPILE);
	{
		GLfloat shadow[] = {0.1, 0.1, 0.3, 0.2};
		glColor4fv(shadow);

		glBegin(GL_QUADS);
		{
			glNormal3f( 0, 1, 0);
			glVertex3f(-size,-size, pnear);
			glVertex3f(-size,-size, pfar);
			glVertex3f( size,-size, pfar);
			glVertex3f( size,-size, pnear);
		}
		glEnd();
	}
	glEndList();

	return tempDL;
}

// This function determines the framerate of the application
void FrameRate()
{
	static int frame = 0;
	static int actualframe = 0;
	static float time = 0;
	static float lasttime = 0;
	static float lastturn = 0;
	char s[50] = "";

	time = glutGet(GLUT_ELAPSED_TIME);

	// Calculate Frame Rate
	if( time - lasttime > 1000 )
	{
		actualframe = frame;
		lasttime = time;
		frame = 0;
	}
	else
	{
		frame++;
	}
	if(time - lastturn > 2000 - (double((CurrentLevel-8)*(CurrentLevel-8)*(CurrentLevel-8))*10))
	{
		lastturn = time;
		theta += .5;
		if(theta > 360.0)
			theta = 0.0;
	}

	sprintf(s,"FPS: %4.0d", actualframe);

	glRasterPos3f(0.7, 0.9, 9.9);

	for(int x = 0; x < 50; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);
}

// detects ball collision
void BallCollisionDetection()
{
	static float time = 0;
	static float lastcalc = 0;

	time = glutGet(GLUT_ELAPSED_TIME);

	// Update ball every five milliseconds
	if( time - lastcalc > 5)
	{

		// Add on the ball velocity vector to position
		// only if not paused
		if( !gPause && !gStop )
		{
			Ball.x += Ballv.x;
			Ball.y += Ballv.y;
			Ball.z += Ballv.z;
		}

		// If ball hits the wall
		if(Ball.x > .95 || Ball.x < -.95)
		{
			if(rand()%2)
			{
				PlaySound("metal2.wav", NULL, SND_ASYNC | SND_FILENAME);
			}
			else
			{
				PlaySound("metal1.wav", NULL, SND_ASYNC | SND_FILENAME);
			}
			if(Ball.x > .95)
				Ball.x = .95;
			else if(Ball.x < -.95)
				Ball.x = -.95;
			Ballv.x *= -1;
		}
		if(Ball.y > .95 || Ball.y < -.95)
		{
			if(rand()%2)
			{
				PlaySound("metal2.wav", NULL, SND_ASYNC | SND_FILENAME);
			}
			else
			{
				PlaySound("metal1.wav", NULL, SND_ASYNC | SND_FILENAME);
			}
			if(Ball.y > .95)
				Ball.y = .95;
			else if(Ball.y < -.95)
				Ball.y = -.95;
			Ballv.y *= -1;
		}
		
		if(Ball.z < .05)
		{
			Ball.z = .05;
			Ballv.z *= -1;
			if(rand()%2)
			{
				PlaySound("metal2.wav", NULL, SND_ASYNC | SND_FILENAME);
			}
			else
			{
				PlaySound("metal1.wav", NULL, SND_ASYNC | SND_FILENAME);
			}
		}

		//If player misses the ball
		if(Ball.z > 13)
		{
			Ballv.z *= -1;
			gStop = true;
			NumberOfLives--;
			gPlayerCombo = 0;
			Ballv.x = initialx;
			Ballv.y = initialy;
			Ballv.z = initialz;
			if(NumberOfLives < 0)
			{
				Reset();
			}
		}

		// Collision detection for user game pad .2
		if((Ball.z < gPadFar+.1 && Ball.z >= gPadFar-.05) 
			   &&(Ball.x > (gPaddleX - 0.25) && Ball.x < (gPaddleX + 0.25)) 
			   &&(-Ball.y > (gPaddleY - 0.25) && -Ball.y < (gPaddleY + 0.25))) 
		{ 
			Ballv.z *= -1;
			Ball.z = gPadFar-.05;
			Ballv.x += 0.1 * DeltaMouseX;
			Ballv.y -= 0.1 * DeltaMouseY;
			
			if(Ballv.x > .04 +(double(CurrentLevel-1)*.01))
				Ballv.x = .04+(double(CurrentLevel-1)*.01);
			if(Ballv.y > .04 +(double(CurrentLevel-1)*.01))
				Ballv.y = .04+(double(CurrentLevel-1)*.01);
			
			Ball.z += Ballv.z;//+(double(CurrentLevel-1)*.01);
			
			if( gHold )
			{
				gStop = true;
				gHold = false;
			}
			else
			{
				gStop = false;
			}
			PlaySound("Paddle.wav", NULL, SND_ASYNC | SND_FILENAME);
			// The ball hit the pad, stop the combo
			gPlayerCombo = 0;
		}

		// check if ball hits a brick
		brickptr cur = BrickLevelOne;
		while( cur != NULL )
		{
			// Ball hits the brick dead on
			if(		( Ball.x > cur->locX-.05 && Ball.x < cur->locX + gBrickWidth +.05 )
				&&	( Ball.y < cur->locY+.05 && Ball.y > cur->locY - (gBrickHeight+.05))
				&&	( Ball.z < cur->locZ-.05 && Ball.z > cur->locZ - (gBrickDepth +.05)) 
				&&	!cur->hit )
			{
				Ballv.z *= -1;
				Ball.x += Ballv.x;
				Ball.y += Ballv.y;
				Ball.z += Ballv.z;
				cur->hit = true;
				// Recalculate the brick display list
				BricksDL = DrawBricks(BrickLevelOne, gBrickHeight, gBrickWidth, gBrickDepth);
				PlaySound("Brick.wav", NULL, SND_ASYNC | SND_FILENAME);
				hit = true;
				// Add onto player's score
				gPlayerCombo++;
				gPlayerScore += 10*(1+gPlayerCombo/2);
				
			}
			cur = cur->next;
		}

		if(Check4Win())
			NextLevel();
		// Save current time
		lastcalc = time;
	}
}

// Set up brick pattern
void InitBrickSetup()
{
	// Seed the randomness
	srand( unsigned( time(NULL) ) );

	GLfloat tempcolor[4];

	brickptr temp;
	for(int z = 8; z > 1; z--)
	{
		tempcolor[0] = float( rand() ) / RAND_MAX;
		tempcolor[1] = float( rand() ) / RAND_MAX;
		tempcolor[2] = float( rand() ) / RAND_MAX;
		tempcolor[3] = 1.0;
		for(int y = 10; y > -10; y -= gBrickHeight*10)
		{
			for(int x = -10; x < 10; x += gBrickWidth*10)
			{
				temp = new brick;
				temp->locX = float(x) / 10.0;
				temp->locY = float(y) / 10.0;
				temp->locZ = float(z);
				temp->hit = false;

				// Select a random color
				temp->color[0] = tempcolor[0];
				temp->color[1] = tempcolor[1];
				temp->color[2] = tempcolor[2];
				temp->color[3] = tempcolor[3];

				temp->next = BrickLevelOne;
				BrickLevelOne = temp;
			}
		}
	}
}

// Draws bricks based on information in linked list
GLuint DrawBricks(brickptr &head, float height, float width, float depth)
{
	GLuint tempDL = glGenLists(1);

	glNewList(tempDL, GL_COMPILE);
	{
		brickptr temp = head;

		while( temp != NULL )
		{
			// Find first brick not hit
			while( temp != NULL && temp->hit )
				temp = temp->next;

			if( temp == NULL )
				break;

			glColor4fv(temp->color);

			// Side facing user
			glBegin(GL_QUADS);
			{
				glNormal3f( 0.0, 0.0, 1.0);
				glVertex3f( temp->locX, temp->locY, temp->locZ);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ);
			}
			glEnd();
			
			// Side away from user
			glBegin(GL_QUADS);
			{
				glNormal3f( 0.0, 0.0,-1.0);
				glVertex3f( temp->locX, temp->locY, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ - depth);
			}
			glEnd();

			// Left side
			glBegin(GL_QUADS);
			{
				glNormal3f( -1.0, 0.0, 0.0);
				glVertex3f( temp->locX, temp->locY, temp->locZ);
				glVertex3f( temp->locX, temp->locY, temp->locZ - depth);
				glVertex3f(	temp->locX, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ);
			}
			glEnd();

			// Right side
			glBegin(GL_QUADS);
			{
				glNormal3f( 1.0, 0.0, 0.0);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ - depth);
			}
			glEnd();

			// Top side
			glBegin(GL_QUADS);
			{
				glNormal3f( 0.0, 1.0, 0.0);
				glVertex3f( temp->locX, temp->locY, temp->locZ - depth);
				glVertex3f( temp->locX, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ - depth);
			}
			glEnd();

			// Bottom side
			glBegin(GL_QUADS);
			{
				glNormal3f( 0.0,-1.0, 0.0);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ);
			}
			glEnd();

			glColor4fv(black);
			//top
			glBegin(GL_LINE_STRIP);
				glVertex3f( temp->locX, temp->locY, temp->locZ+.001);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ+.001);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ+.001);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ+.001);
			glEnd();
			//top
			glBegin(GL_LINE_STRIP);
				glVertex3f( temp->locX, temp->locY, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ - depth);
			glEnd();
			//top
			glBegin(GL_LINE_STRIP);
				glVertex3f( temp->locX, temp->locY, temp->locZ);
				glVertex3f( temp->locX, temp->locY, temp->locZ - depth);
				glVertex3f(	temp->locX, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX, temp->locY - height, temp->locZ);
			glEnd();
			//top
			glBegin(GL_LINE_STRIP);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY - height, temp->locZ - depth);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ - depth);
			glEnd();
			//top
			glBegin(GL_LINE_STRIP);
				glVertex3f( temp->locX, temp->locY, temp->locZ - depth);
				glVertex3f( temp->locX, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ - depth);
			glEnd();
			//bot
			glBegin(GL_LINE_STRIP);
				glVertex3f( temp->locX, temp->locY, temp->locZ - depth);
				glVertex3f( temp->locX, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ);
				glVertex3f( temp->locX + width, temp->locY, temp->locZ - depth);
			glEnd();

			temp = temp->next;
		}
	}
	glEndList();

	return tempDL;
}

// Display list for the game ball
GLuint CreateBall()
{
	GLuint tempDL = glGenLists(1);

	glNewList(tempDL, GL_COMPILE);
	{
		glColor4fv(white);
		glutSolidSphere(.05, 32, 32);
	}
	glEndList();

	return tempDL;
}

void MusicSelector(int selection)
{
	loading = true;
	myDisplay();
	switch(selection)
	{
		case 1:
			PlayMidi("audio`.mid");
			break;
		case 2:
			PlayMidi("audio1.mid");
			break;
		case 3:
			PlayMidi("audio2.mid");
			break;
		case 4:
			PlayMidi("audio3.mid");
			break;
		case 5:
			PlayMidi("audio4.mid");
			break;
		case 6:
			PlayMidi("audio5.mid");
			break;
		case 7:
			PlayMidi("audio6.mid");
			break;
		case 8:
			PlayMidi("audio7.mid");
			break;
		case 9:
			PlayMidi("audio8.mid");
			break;
		case 10:
			PlayMidi("audio9.mid");
			break;
		case 11:
			PlayMidi("audio0.mid");
			break;
		case 12:
			PlayMidi("audio-.mid");
			break;
		case 13:
			PlayMidi("audio=.mid");
			break;
		case 14:
			PlayMidi("audioq.mid");
			break;
		case 15:
			PlayMidi("audiow.mid");
			break;
		case 16:
			PlayMidi("audioe.mid");
			break;
		case 17:
			PlayMidi("audior.mid");
			break;
		case 18:
			PlayMidi("audiot.mid");
			break;
		case 19:
			PlayMidi("audioy.mid");
			break;
	}
	loading = false;
}

void DisplayMenu()
{
	glEnable(GL_LIGHT1);
	glColor3fv(red);
	char s[78] = "";
	//Displays the opening menu and draws ball not based on timer
	sprintf(s,"Welcome to another take at a brick breaker game!");
	glRasterPos3f(-0.5, 0.8, 9.9);
	for(int x = 0; x < 78; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);

	sprintf(s,"Made by Jeremy Paskali & Ian McCloy                 ");
	glRasterPos3f(-0.4, -0.8, 9.9);
	for(x = 0; x < 78; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);

	glColor3fv(green);
	sprintf(s,"Controls:                                           ");
	glRasterPos3f(-0.7, 0.5, 9.9);
	for(x = 0; x < 78; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);

	sprintf(s,"Pause                 P                             ");
	glRasterPos3f(-0.6, 0.4, 9.9);
	for(x = 0; x < 78; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);

	sprintf(s,"Cheat                 G                             ");
	glRasterPos3f(-0.6, 0.3, 9.9);
	for(x = 0; x < 78; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);

	sprintf(s,"Play Midi             \"~\" -> \"+\"     &     shift   +   \"~\" -> \"5\"");
	glRasterPos3f(-0.6, 0.2, 9.9); 
	for(x = 0; x < 78; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[x]);





	glLightfv(GL_LIGHT0, GL_POSITION, light_position);


	for(int i = 0; i < 25; i++)
	{
		Ball.x += Ballv.x;
		Ball.y += Ballv.y;

		if(Ball.x > .95 || Ball.x < -.95)
			Ballv.x *= -1;
		if(Ball.y > .95 || Ball.y < -.95)
			Ballv.y *= -1;

		glPushMatrix();
		{
			glTranslated(Ball.x, Ball.y, 9.8);
			glCallList(DaBall);
		}
		glPopMatrix();
	}
}

void Reset()
{
	gMenu = true;			//Draw the menu istead of the game
	gPlayerScore = 0;		//Reset the score
	Ballv.x = Menux;		//Move the ball at a diffrent speed
	Ballv.y = Menuy;		
	CurrentLevel = 0;		//Reset level
	initialx = .0000;		
	initialy = .0001;
	initialz = -.06;
	NumberOfLives = 4;		//Reset Lives
}	
void ResetBricks()
{
	//This function will pick a certain number of bricks to make per level
	//Then set the hit values of the others to true so they dont display.
	//By level 9 all bricks are displayed
	srand( unsigned( time(NULL) ) );
	brickptr cur = BrickLevelOne;
	bool brick;
	
	int bricks = double(CurrentLevel)*11.2;
	if(bricks > 56)
		bricks = 56;

	while( cur != NULL )
	{
		brick = bool(rand()%2);
		if(!brick)
			if(bricks > 0)
				bricks --;
			else
				brick = true;
		if(CurrentLevel >= 9)
			brick = false;
		
		cur->hit = brick;
		cur->color[0] = float( rand() ) / RAND_MAX;
		cur->color[0] = float( rand() ) / RAND_MAX;
		cur->color[0] = float( rand() ) / RAND_MAX;
		cur->color[0] = 1.0;
		cur = cur->next;
	}
	BricksDL = DrawBricks(BrickLevelOne, gBrickHeight, gBrickWidth, gBrickDepth);
}

void NextLevel()
{
	MusicSelector(1+(rand()%19));
	gStop = true;
	CurrentLevel++;
	initialx = double(CurrentLevel-1)*.0001;
	initialy = double(CurrentLevel  )*.0001;
	initialz = -(.06 + (double(CurrentLevel-1)*.001));
	Ballv.x = initialx;
	Ballv.y = initialy;
	Ballv.z = initialz;
	ResetBricks();
	gPause = false;
	theta = 0.0;

	//Every 4 levels makes the level dark to keep it interesting
	if(CurrentLevel%4 == 0)
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_darkness);
	else
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
}

bool Check4Win()
{
	//If all bricks are hit return true
	brickptr cur = BrickLevelOne;
	while( cur != NULL )
	{
		if(!cur->hit )
			return false;
		cur = cur->next;
	}
	return true;
}
void LoadScreen()
{
	//Little bit of random color for fun
	glEnable(GL_LIGHT1);
	float one   = float(rand()-.1)/RAND_MAX;
	float two   = float(rand()+.1)/RAND_MAX;
	float three = float(rand()+.1)/RAND_MAX;

	glColor3d(one,two,three);

	char load[10] = "";
	sprintf(load,"Loading...");
	glRasterPos3f(-0.1, 0, 9.9);
	for(int x = 0; x < 10; x++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, load[x]);
	
	glDisable(GL_LIGHT1);

}
