/**********************************************************
Camera class created by Jeremy Paskali
Now includes point and vector classes

// Assisted by...
// Ed Nelson
// Computer Graphics
// Fall 2003
// From Computer Graphics with OpenGL, by FS Hill JR

camera.h
**********************************************************/
#include <windows.h>
#include <math.h>
#include <gl/Gl.h>
#include <gl/Glu.h>
#include <gl/glut.h>
#include <stdio.h>

class Point3
{
// from page 841, Appendix 3
public:
	float x,y,z;
	void set(float dx, float dy, float dz) {x = dx; y=dy; z=dz;}
	void set(Point3 &p) {x = p.x; y = p.y; z = p.z;}
	Point3( float xx, float yy, float zz) {x = xx; y = yy; z = zz;}
	Point3() {x = y= z = 0;}
	void build4tuple(float v[])
	{ 
		//load 4-tuple with this color v[3] = 1 for homogeneous
		v[0] = x; v[1] = y; v[2] = z; v[3] = 1.0f;
	}
};

class Vector3
{
// from page 841, Appendix 3
public:
	float x,y,z;

	void set(float dx, float dy, float dz) {x = dx; y = dy; z = dz;}
	void set(Vector3 &v) {x = v.x; y = v.y; z = v.z;}
	void flip() {x = -x; y = -y; z = -z;} // reverse the vector
	void setDiff(Point3& a, Point3& b) //set to difference a - b
		{x = a.x - b.x; y = a.y - b.y; z = a.z - b.z;}
	Vector3(float xx, float yy, float zz) {x = xx; y = yy; z = zz;}
	Vector3(Vector3& v){x = v.x; y = v.y; z = v.z;}
	Vector3() {x = y = z = 0;} //default constructor

	/// Provide these methods
	void normalize(); //adjust this vector to unit length
	Vector3 cross (Vector3 b); // return this cross b
	float dot (Vector3 b);  // return this dotted with b
};

class CCamera
{
public:

	CCamera();  //default constructor
	void set (Point3 eye, Point3 look, Vector3 up); //like gluLookAt()
	void move(int direction);	// move forwards or backwards along vector n
	void roll(float angle);  //roll camera
	void pitch(float angle);  //increase pitch
	void yaw(float angle);   //yaw it
	void slide(int direction); //slide it
	void rise(int direction); // rise or sink
	void setShape(float vAng, float asp, float nearD, float farD);
	void modifyShape( float vAng, float asp, float nearD, float farD);

	Point3 eye;
	Vector3 u,v,n;

private:
	double viewAngle, aspect, nearDist, farDist; // view volume shape
	void setModelViewMatrix(); //Tell Open GL where the camera is
};

class VertexID{
	public:
		int vertIndex;  //index of vertex in the vertex list
		int normIndex;  //index of this vertiex's normal
};

class Face {
	public:
		int nVerts;
		VertexID vert[8];
		Face(){nVerts = 0;};
		~Face(){nVerts=0;}
};

class Mesh {
	private:
		int numVerts;
		Point3 pt[400];
		int numNormals;
		Vector3 norm[400];
		int numFaces;
		Face face[900];

	public:
		Mesh(){numVerts = 0; numNormals = 0; numFaces = 0;};
		~Mesh(){numVerts =  numNormals = numFaces = 0;};
		void draw(int wire);
		GLuint createDL(int wire);

		// Ed Nelson: I created this method
		void makeSphere( float radius );
		void makePyramid(float percent);

};
